unit NEXCommandLineMain;

interface

uses
  NEXTypes, NEXLowLevel, NEXUtils, NEXObjs, Sysutils, Classes, Math;

implementation

function CmdInitPlayer : boolean;
begin
  Result := false;
  // Initialise the connection
  NexDoLog('Connecting to player ...');
  if InitPlayer2 <> $0055 then
  begin
    NexDoLog('Unable to connect to player. Please make sure the NEX is turned on and connected');
    exit;
  end;
  SelectBank(4);  // Returns $FFFF (invalid bank, as I don't have an extension card)
  InitPlayer2;
  SelectBank(0);  // Returns $EC73 (valid bank, chunk size = 32 sectors)
  Result := true;
end;

procedure CmdProcessCmd;
var
  nfs         : TNexFilesystem;
  Stream1     : TNexStream;
  OutStream   : TFileStream;
  FileLoop    : integer;
  pFI         : PFileEntry;
  TmpCode     : integer;
  TmpBlock    : pointer;
  NumDone     : integer;
  NumToDo     : integer;
  NumPrinted  : integer;
begin
  if ParamCount > 0 then
  begin
    if LowerCase(ParamStr(1)) = 'list' then
    begin
      if ParamCount = 1 then
      begin
        if not CmdInitPlayer then exit;
        writeln('Downloading filesystem information ...');
        nfs := TNexFilesystem.Create;
        writeln('Memory free/total: ' + IntToStr(nfs.FreeBytes shr 10) + '/' +
          IntToStr(nfs.TotalBytes shr 10) + ' kb');
        for FileLoop := 0 to MAXSONGS - 1 do
        begin
          pFI := NFS.GetFileEntry(FileLoop);
          if pFI.SongNumber <> $FF then
          begin
            writeln('Slot ' + IntToStr(FileLoop) + ': ' + pFI.FileName +
              ' (file timestamp = ' + NEXDateToStr(pFI^) + ')');
            writeln('  Title : ' + pFI.SongTitle);
            writeln('  Artist: ' + pFI.SongArtist);
            writeln('  Info  : ' + IntToStr(pFI.Filesize shr 10) + 'kb, ' +
              IntToStr(GetBitrate(pFI^)) + 'kbps, duration ' +
              IntToStr(pFI^.Duration) + ' seconds');
          end;
        end;
        nfs.Free;
        exit;
      end;
      writeln('Too many parameters');
      writeln('');
    end
    else if LowerCase(ParamStr(1)) = 'get' then
    begin
      if ParamCount = 3 then
      begin
        val(ParamStr(2), FileLoop, TmpCode);
        if (TmpCode = 0) and (FileLoop >= 0) and (FileLoop < MAXSONGS) then
        begin
          if not CmdInitPlayer then exit;
          writeln('Downloading filesystem information ...');
          nfs := TNexFilesystem.Create;
          pFI := NFS.GetFileEntry(FileLoop);
          if pFI.SongNumber <> $FF then
          begin
            Stream1 := TNexStream.Create(nfs, 0);
            try
              OutStream := TFileStream.Create(ParamStr(3), fmCreate or fmShareDenyNone);
            except
              on E:EFCreateError do
              begin
                Stream1.Free;
                writeln('Could not create file ''' + ParamStr(3) + '''');
                exit;
              end;
            end;
            GetMem(TmpBlock, SECTORSIZE*SECTORSPERBLOCK);
            NumToDo := Stream1.Size;
            Write('Downloading file: ');
            Write('|' + StringOfChar(' ', 40) + '|');
            Write(StringOfChar(#8, 41));
            NumPrinted := 0;
            while NumToDo > 0 do
            begin
              if NumToDo > SECTORSIZE*SECTORSPERBLOCK then
                NumDone := Stream1.Read(TmpBlock^, SECTORSIZE*SECTORSPERBLOCK)
              else
                NumDone := Stream1.Read(TmpBlock^, NumToDo);
              OutStream.Write(TmpBlock^, NumDone);
              dec(NumToDo, NumDone);
              while NumPrinted < trunc((1 - NumToDo/Stream1.Size)*40) do
              begin
                Write('=');
                inc(NumPrinted);
              end;
            end;
            Write('|');
            Writeln(StringOfChar(#8, 42) + StringOfChar(' ', 42) +
              StringOfChar(#8, 42) + 'Completed');
            OutStream.Free;
            Stream1.Free;
          end
          else
          begin
            writeln('No such song on player');
            writeln('');
          end;
          nfs.free;
          exit;
        end
        else
        begin
          writeln('Invalid song number');
          writeln('');
        end;
      end
      else
      begin
        writeln('Incorrect number of parameters');
        writeln('');
      end;
    end
    else
    begin
      writeln('Command ' + ParamStr(1) + ' not recognised');
      writeln('');
    end;
  end;

  writeln('nexcmdline <command> [parameter] [parameter] ...');
  writeln('Commands:');
  writeln('  list           = list songs on the player.');
  writeln('  get <n> <file> = download a song from the player. n = song number. The mp3 is');
  writeln('                   stored in the specified file.');
  writeln('  put <file>     = store a song on the player. NOT IMPLEMENTED YET.');
end;

initialization
  CmdProcessCmd;

end.
