unit NEXTypes;

interface

type
  uint8   = byte;
  uint16  = word;
  uint32  = longword;
  sint8   = shortint;
  sint16  = smallint;
  sint32  = longint;

const
  MAXSONGS        = 60;
  NUMFATITEMS     = 4096;
  SECTORSIZE      = $210;
  SECTORSPERBLOCK = 64;

type
  TFileEntry = packed record
    SongNumber  : uint16;
    FileName    : array[0..15] of char;
    SongTitle   : array[0..45] of char;
    SongArtist  : array[0..45] of char;
    Day         : uint8;
    Month       : uint8;
    YearHigh    : uint8;
    YearLow     : uint8;
    Hour        : uint8;
    Minute      : uint8;
    Filesize    : uint32;
    MP3FrameHdr : uint16;
    Duration    : uint16;
    BlockStart  : uint16;
    Unknown2    : uint16;
  end;
  PFileEntry = ^TFileEntry;

  TFileTable = packed array[0..MAXSONGS - 1] of TFileEntry;

  TSector0F = packed record
    Unknowns        : array[0..5] of uint16;
    Filler1         : array[0..227] of uint8;
    Unknown1        : uint16;
    FirstFree       : uint16;
    LastBlock       : uint16;
    Unknown2        : uint16;
    Unknown3        : uint16;
    FATSectorCount  : uint16;
    Filler2         : array[0..259] of uint8;
  end;

  TFATItem = packed record
    NextFAT : uint16;
    PrevFAT : uint16;
  end;

  TFATTable = packed array[0..NUMFATITEMS-1] of TFATItem;

  TNEXLowMemory = packed record
    FileTable : TFileTable;
    Sector0F  : TSector0F;
    FATTable  : TFATTable;
    Pattern   : array[0..4095] of uint8;
    Filler    : array[0..4095] of uint8;
  end;

  TLogFunction = procedure(LogStr : string) of object;

var
  LogFunc : TLogFunction;

procedure NexDoLog(LogStr : string);

implementation

procedure NexDoLog(LogStr : string);
begin
  if Assigned(LogFunc) then
    LogFunc(LogStr)
  else
  begin
    {$IFDEF CONSOLE}
//    writeln(LogStr);
    {$ENDIF}
  end;
end;

end.
 